/****************************************************************************
** Meta object code from reading C++ file 'bomberwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../bomber-26.04.2/src/bomberwidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'bomberwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16BomberGameWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto BomberGameWidget::qt_create_metaobjectdata<qt_meta_tag_ZN16BomberGameWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "BomberGameWidget",
        "gameOver",
        "",
        "levelChanged",
        "level",
        "scoreChanged",
        "score",
        "livesChanged",
        "lives",
        "timeChanged",
        "time",
        "stateChanged",
        "BomberGameWidget::State",
        "state",
        "closeGame",
        "newGame",
        "setPaused",
        "paused",
        "settingsChanged",
        "setSoundsEnabled",
        "enable",
        "setSuspended",
        "value",
        "onDropBomb",
        "tick",
        "onBombHit",
        "onLevelCleared",
        "onPlaneCrashed"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'gameOver'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'levelChanged'
        QtMocHelpers::SignalData<void(unsigned int)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 4 },
        }}),
        // Signal 'scoreChanged'
        QtMocHelpers::SignalData<void(unsigned int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 6 },
        }}),
        // Signal 'livesChanged'
        QtMocHelpers::SignalData<void(unsigned int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 8 },
        }}),
        // Signal 'timeChanged'
        QtMocHelpers::SignalData<void(unsigned int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 10 },
        }}),
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void(BomberGameWidget::State)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'closeGame'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'newGame'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setPaused'
        QtMocHelpers::SlotData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'settingsChanged'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setSoundsEnabled'
        QtMocHelpers::SlotData<void(bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 20 },
        }}),
        // Slot 'setSuspended'
        QtMocHelpers::SlotData<void(bool)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 22 },
        }}),
        // Slot 'onDropBomb'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'tick'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onBombHit'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onLevelCleared'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onPlaneCrashed'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<BomberGameWidget, qt_meta_tag_ZN16BomberGameWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject BomberGameWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16BomberGameWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16BomberGameWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16BomberGameWidgetE_t>.metaTypes,
    nullptr
} };

void BomberGameWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<BomberGameWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->gameOver(); break;
        case 1: _t->levelChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 2: _t->scoreChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 3: _t->livesChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 4: _t->timeChanged((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 5: _t->stateChanged((*reinterpret_cast<std::add_pointer_t<BomberGameWidget::State>>(_a[1]))); break;
        case 6: _t->closeGame(); break;
        case 7: _t->newGame(); break;
        case 8: _t->setPaused((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->settingsChanged(); break;
        case 10: _t->setSoundsEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->setSuspended((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->onDropBomb(); break;
        case 13: _t->tick(); break;
        case 14: _t->onBombHit(); break;
        case 15: _t->onLevelCleared(); break;
        case 16: _t->onPlaneCrashed(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (BomberGameWidget::*)()>(_a, &BomberGameWidget::gameOver, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (BomberGameWidget::*)(unsigned int )>(_a, &BomberGameWidget::levelChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (BomberGameWidget::*)(unsigned int )>(_a, &BomberGameWidget::scoreChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (BomberGameWidget::*)(unsigned int )>(_a, &BomberGameWidget::livesChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (BomberGameWidget::*)(unsigned int )>(_a, &BomberGameWidget::timeChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (BomberGameWidget::*)(BomberGameWidget::State )>(_a, &BomberGameWidget::stateChanged, 5))
            return;
    }
}

const QMetaObject *BomberGameWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BomberGameWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16BomberGameWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QGraphicsView::qt_metacast(_clname);
}

int BomberGameWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void BomberGameWidget::gameOver()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void BomberGameWidget::levelChanged(unsigned int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void BomberGameWidget::scoreChanged(unsigned int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void BomberGameWidget::livesChanged(unsigned int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void BomberGameWidget::timeChanged(unsigned int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void BomberGameWidget::stateChanged(BomberGameWidget::State _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
